package org.hyperscala.web

import org.hyperscala.html._
import scala.collection.JavaConversions._
import site.Webpage
import tag.Form
import org.jboss.netty.buffer.ChannelBuffer
import org.jboss.netty.handler.codec.http.QueryStringDecoder
import org.jboss.netty.util.CharsetUtil
import org.hyperscala.event.SubmitEvent
import org.powerscala.hierarchy.ChildLike
import org.powerscala.TypeFilteredIterator

/**
 * @author Matt Hicks <mhicks@powerscala.org>
 */
trait FormSupport extends Webpage {
  override protected def processPost(content: ChannelBuffer) = {
    val decoder = new QueryStringDecoder("?%s".format(content.toString(CharsetUtil.UTF_8)))
    var form: Form = null
    decoder.getParameters.foreach {
      case (key, values) => {
        html.byName[HTMLTag](key).headOption match {
          case Some(t) if (t.renderable) => {      // Only apply to tags that are rendered to the page
            if (form == null) {
              TypeFilteredIterator[Form](ChildLike.selfAndAncestors(t)).toStream.headOption match {
                case None => warn("WARNING: Unable to find form for %s".format(key)) // Odd, but not impossible
                case Some(f) => form = f
              }
            }
            val v = values.toArray.map(o => o.toString)
            t match {
              case button: tag.Button => button.submitEvent.fire(new SubmitEvent(button))
              case _ => t.formValue := v.head
            }
          }
          case _ => warn("Unable to find %s = %s".format(key, values.head))
        }
      }
    }
    if (form != null) {
      form.submitEvent.fire(new SubmitEvent(form))
    }
  }
}
